@@ -8,9 +8,11 @@ import io
 import os
 import stat
 import socket
+import sys
 
 import six
 import paramiko
+from cached_property import threaded_cached_property as cached_property
 
 from .. import errors
 from ..base import FS
@@ -56,7 +58,7 @@ class SSHFS(FS):
         'invalid_path_chars': '\0',
         'network': True,
         'read_only': False,
-        'thread_safe': True,
+        'thread_safe': sys.version_info[:2] > (3, 4),
         'unicode_paths': True,
         'virtual': False,
     }
@@ -76,10 +78,9 @@ class SSHFS(FS):
             pass
         return ssh_config
 
-
     def __init__(self, host, user=None, passwd=None, pkey=None, timeout=10,
                  port=22, keepalive=10, compress=False,
-                 config_path='~/.ssh/config'):  # noqa: D102
+                 config_path='~/.ssh/config', **kwargs):  # noqa: D102
         super(SSHFS, self).__init__()
 
         # Attempt to get a configuration for the given host
@@ -93,24 +94,30 @@ class SSHFS(FS):
         self._host = host = config.get('hostname')
         self._port = port = int(config.get('port', port))
         self._client = client = paramiko.SSHClient()
-        self._locale = None
+        self._timeout = timeout
 
         try:
-
             # TODO: add more options
             client.load_system_host_keys()
             client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+            argdict = {
+                "pkey": pkey,
+                "key_filename": keyfile,
+                "look_for_keys": True if (pkey and keyfile) is None else False,
+                "compress": compress,
+                "timeout": timeout
+            }
+
+            argdict.update(kwargs)
+
             client.connect(
                 socket.gethostbyname(host), port, user, passwd,
-                pkey=pkey, key_filename=keyfile,
-                look_for_keys=True if (pkey and keyfile) is None else False,
-                compress=compress, timeout=timeout
+                **argdict
             )
 
             if keepalive > 0:
                 client.get_transport().set_keepalive(keepalive)
             self._sftp = client.open_sftp()
-            self._platform = None
 
         except (paramiko.ssh_exception.SSHException,            # protocol errors
                 paramiko.ssh_exception.NoValidConnectionsError, # connexion errors
@@ -198,14 +205,18 @@ class SSHFS(FS):
         _mode.validate_bin()
 
         with self._lock:
-            if _mode.exclusive and self.exists(_path):
-                raise errors.FileExists(path)
+            if _mode.exclusive:
+                if self.exists(_path):
+                    raise errors.FileExists(path)
+                else:
+                    _mode = Mode(''.join(set(mode.replace('x', 'w'))))
             elif _mode.reading and not _mode.create and not self.exists(_path):
                 raise errors.ResourceNotFound(path)
             elif self.isdir(_path):
                 raise errors.FileExpected(path)
             with convert_sshfs_errors('openbin', path):
-                handle = self._sftp.open(
+                _sftp = self._client.open_sftp()
+                handle = _sftp.open(
                     _path,
                     mode=_mode.to_platform_bin(),
                     bufsize=buffering
@@ -262,43 +273,12 @@ class SSHFS(FS):
             if 'permissions' in access:
                 self._chmod(path, access['permissions'].mode)
 
-    @property
+    @cached_property
     def platform(self):
         """The platform the server is running on.
 
         Returns:
-            fs.sshfs.enums.Platform: the platform of the remote server.
-        """
-        if self._platform is None:
-            self._platform = self._guess_platform()
-        return self._platform
-
-    @property
-    def locale(self):
-        """The locale the server is running on.
-
-        Returns:
-            str: the locale of the remote server.
-        """
-        if self._locale is None:
-            self._locale = self._guess_locale()
-        return self._locale
-
-    def _exec_command(self, cmd):
-        """Run a command on the remote SSH server.
-
-        Returns:
-            bytes: the output of the command, if it didn't fail
-            None: if the error pipe of the command was not empty
-        """
-        _, out, err = self._client.exec_command(cmd)
-        return out.read().strip() if not err.read().strip() else None
-
-    def _guess_platform(self):
-        """Guess the platform of the remove server.
-
-        Returns:
-            fs.sshfs.enums.Platform: the guessed platform.
+            str: the platform of the remote server, as in `sys.platform`.
         """
         uname_sys = self._exec_command("uname -s")
         sysinfo = self._exec_command("sysinfo")
@@ -315,11 +295,12 @@ class SSHFS(FS):
             return "win32"
         return "unknown"
 
-    def _guess_locale(self):
-        """Guess the locale of the remote server.
+    @cached_property
+    def locale(self):
+        """The locale the server is running on.
 
         Returns:
-            str: the guessed locale.
+            str: the locale of the remote server if detected, or `None`.
         """
         if self.platform in ("linux", "darwin", "freebsd"):
             locale = self._exec_command('echo $LANG')
@@ -327,6 +308,16 @@ class SSHFS(FS):
                 return locale.split(b'.')[-1].decode('ascii').lower()
         return None
 
+    def _exec_command(self, cmd):
+        """Run a command on the remote SSH server.
+
+        Returns:
+            bytes: the output of the command, if it didn't fail
+            None: if the error pipe of the command was not empty
+        """
+        _, out, err = self._client.exec_command(cmd, timeout=self._timeout)
+        return out.read().strip() if not err.read().strip() else None
+
     def _make_info(self, name, stat_result, namespaces):
         """Create an `Info` object from a stat result.
         """
